@@ -80,8 +80,13 @@ Type RCM(objective_function<Type> *obj) {
   DATA_MATRIX(prior_dist); // Distribution of priors for R0, h, M, q (rows), columns indicate parameters of distribution calculated in R (see RCM_prior fn)
   
   DATA_INTEGER(nll_gr);    // Whether to ADREPORT annual likelihoods
+  
+  DATA_IMATRIX(est_vul);  // Indicates if fishery selectivity parameters are estimated. Then, will add vague priors to aid convergence
+  DATA_IMATRIX(est_ivul); // Same but for index selectivity parameters
+  
   DATA_IVECTOR(est_early_rec_dev); // Indicates years in which log_early_rec_dev are estimated. Then, lognormal bias correction estimates are added..
   DATA_IVECTOR(est_rec_dev); // Indicates years in which log_rec_dev are estimated.
+  
 
   PARAMETER(R0x);                       // Unfished recruitment
   PARAMETER(transformed_h);             // Steepness
@@ -123,7 +128,7 @@ Type RCM(objective_function<Type> *obj) {
   vector<Type> LFS(nsel_block);
   vector<Type> L5(nsel_block);
   vector<Type> Vmaxlen(nsel_block);
-  array<Type> vul = calc_vul(vul_par, vul_type, len_age, LFS, L5, Vmaxlen, Linf, nfleet, sel_block, nsel_block, prior);
+  array<Type> vul = calc_vul(vul_par, vul_type, len_age, LFS, L5, Vmaxlen, Linf, nfleet, sel_block, nsel_block, prior, est_vul);
 
   vector<Type> q_effort(nfleet);
   vector<Type> CV_msize(nfleet);
@@ -171,7 +176,7 @@ Type RCM(objective_function<Type> *obj) {
   Brec /= E0_SR;
   
   ////// During time series year = 1, 2, ..., n_y
-  vector<matrix<Type> > ALK(n_y);
+  vector<matrix<Type> > PLA(n_y);
   matrix<Type> N(n_y+1, n_age);
 
   vector<Type> C_eq_pred(nfleet);
@@ -241,8 +246,8 @@ Type RCM(objective_function<Type> *obj) {
 
   // Loop over all other years
   for(int y=0;y<n_y;y++) {
-    // Calculate this year's age-length key (ALK) and F
-    if(Type(n_age) != Linf) ALK(y) = generate_ALK(lbin, len_age, SD_LAA, n_age, nlbin, y);
+    // Calculate this year's probability of length-at-age (PLA) and F
+    if(Type(n_age) != Linf) PLA(y) = generate_PLA(lbin, len_age, SD_LAA, n_age, nlbin, y);
     if(condition == "catch") {
       for(int ff=0;ff<nfleet;ff++) {
         if(y != yind_F(ff)) {
@@ -272,8 +277,8 @@ Type RCM(objective_function<Type> *obj) {
 
         if(Type(n_age) != Linf) {
           for(int len=0;len<nlbin;len++) {
-            CALpred(y,len,ff) += CAAtrue(y,a,ff) * ALK(y)(a,len);
-            MLpred(y,ff) += CAAtrue(y,a,ff) * ALK(y)(a,len) * lbinmid(len);
+            CALpred(y,len,ff) += CAAtrue(y,a,ff) * PLA(y)(a,len);
+            MLpred(y,ff) += CAAtrue(y,a,ff) * PLA(y)(a,len) * lbinmid(len);
           }
         }
         for(int aa=0;aa<n_age;aa++) CAApred(y,aa,ff) += CAAtrue(y,a,ff) * age_error(a,aa); // a = true, aa = observed ages
@@ -315,7 +320,7 @@ Type RCM(objective_function<Type> *obj) {
   IN.setZero();
   Itot.setZero();
 
-  array<Type> ivul = calc_ivul(ivul_par, ivul_type, len_age, iLFS, iL5, iVmaxlen, Linf, mat, vul, prior);
+  array<Type> ivul = calc_ivul(ivul_par, ivul_type, len_age, iLFS, iL5, iVmaxlen, Linf, mat, vul, prior, est_ivul);
   vector<Type> q(nsurvey);
   for(int sur=0;sur<nsurvey;sur++) {
     for(int y=0;y<n_y;y++) {
@@ -327,7 +332,7 @@ Type RCM(objective_function<Type> *obj) {
 
         if(I_units(sur)) Itot(y,sur) += IAAtrue(y,a,sur) * wt(y,a); // Biomass vulnerable to survey
         if(Type(n_age) != Linf && IAL_n.col(sur).sum() > 0) { // Predict survey length comps if there are data
-          for(int len=0;len<nlbin;len++) IALpred(y,len,sur) += IAAtrue(y,a,sur) * ALK(y)(a,len);
+          for(int len=0;len<nlbin;len++) IALpred(y,len,sur) += IAAtrue(y,a,sur) * PLA(y)(a,len);
         }
       }
     }
@@ -336,7 +341,7 @@ Type RCM(objective_function<Type> *obj) {
   }
 
   // Calc likelihood and parameter prior
-  prior -= calc_prior(use_prior, prior_dist, R0, h, SR_type == "BH", log_M, q);
+  prior -= calc_prior(use_prior, prior_dist, R0x, h, SR_type == "BH", log_M, q, rescale);
   
   array<Type> nll_fleet(n_y,nfleet,5);
   array<Type> nll_index(n_y,nsurvey,3);
@@ -411,10 +416,10 @@ Type RCM(objective_function<Type> *obj) {
   }
 
   for(int y=0;y<n_y;y++) {
-    if(est_rec_dev(y)) nll_log_rec_dev -= dnorm_(log_rec_dev(y), Type(0), tau, true);
+    if(est_rec_dev(y) == 1) nll_log_rec_dev -= dnorm_(log_rec_dev(y), Type(0), tau, true);
   }
   for(int a=0;a<n_age-1;a++) {
-    if(est_early_rec_dev(a)) nll_log_rec_dev -= dnorm_(log_early_rec_dev(a), Type(0), tau, true);
+    if(est_early_rec_dev(a) == 1) nll_log_rec_dev -= dnorm_(log_early_rec_dev(a), Type(0), tau, true);
   }
 
   Type nll = nll_fleet.sum() + nll_index.sum();
@@ -461,7 +466,6 @@ Type RCM(objective_function<Type> *obj) {
   REPORT(EPR0_SR);
   REPORT(CR_SR);
 
-  if(nll_fleet.col(3).sum() != 0 || nll_index.col(2).sum() != 0 || ((nll_fleet.col(4).sum() != 0) & (msize_type == "length"))) REPORT(ALK);
   REPORT(N);
   REPORT(CAApred);
   REPORT(CALpred);
