@@ -7,6 +7,7 @@ Type SP_F(Type U_start, Type C_hist, Type MSY, Type K, Type n, Type nterm, Type
           vector<Type> &Cpred, vector<Type> &B, int y, Type &penalty) {
   Type F;
   Type F_out;
+  Type B_out;
   if(nstep > 1) {
     F = -log(1 - U_start);
     for(int i=0;i<n_itF;i++) {
@@ -33,15 +34,16 @@ Type SP_F(Type U_start, Type C_hist, Type MSY, Type K, Type n, Type nterm, Type
       Catch += F_out * B_next * dt;
       B_next += SP;
     }
-    B(y+1) = B_next;
+    B_out = B_next;
     Cpred(y) = Catch;
   } else {
     F = C_hist/B(y);
     F_out = CppAD::CondExpLt(1 - F, Type(0.025), 1 - posfun(1 - F, Type(0.025), penalty), F);
     Cpred(y) = F_out * B(y);
-    B(y+1) = B(y) + CppAD::CondExpEq(n, Type(1), -exp(Type(1.0)) * MSY * B(y) / K * log(B(y)/K),
-      nterm/(n-1) * MSY * (B(y)/K - pow(B(y)/K, n))) - F_out * B(y);
+    B_out = B(y) + CppAD::CondExpEq(n, Type(1), -exp(Type(1.0)) * MSY * B(y) / K * log(B(y)/K),
+              nterm/(n-1) * MSY * (B(y)/K - pow(B(y)/K, n))) - F_out * B(y);
   }
+  B(y+1) = CppAD::CondExpGt(B_out, Type(1e-8), B_out, posfun(B_out, Type(1e-8), penalty));
   return F_out;
 }
 
